VERSION 5.00
Begin VB.UserControl Rocket 
   ClientHeight    =   3600
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   PropertyPages   =   "Rocket.ctx":0000
   ScaleHeight     =   3600
   ScaleWidth      =   4800
   Begin VB.CommandButton Rocket 
      Caption         =   "Bitwise 1"
      Height          =   855
      Left            =   1080
      TabIndex        =   0
      Top             =   1080
      Width           =   1815
   End
End
Attribute VB_Name = "Rocket"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "PropPageWizardRun" ,"Yes"
 '
 ' Visual Basic Sample Code provided by BITWISE MAGAZINE
 '
 '      http://www.bitwisemag.com
 '
 ' This code and information is provided "as is" without warranty of
 ' any kind, either expressed or implied, including but not limited
 ' to the implied warranties of merchantability and/or fitness for a
 ' particular purpose.
 '
 ' You may use and modify this code for your personal use only.
 ' You may also redistribute this code providing that
 ' a) No fee is charged
 ' b) This copyright notice is retained in the source code.
 '
 ' You may not use this code for commercial purposes
 ' without the express permission of the copyright holder
 '
 ' Copyright (c) 2006 Dermot Hogan
 '
 '
Option Explicit

' m = rocket mass
' vh = vertical velocity
' vh = horizontal velocity
' a = instantaneous accelaration
Dim m, vh, vv, a, alpha, pi As Double
Const g = 9.81

Const m_def_FuelVelocity = 600
Const m_def_BurnRate = 10
Const m_def_InitialAngle = 90
Const m_def_Payload = 200
Const m_def_Fuel = 400
'Property Variables:
Dim m_FuelVelocity As Double
Dim m_BurnRate As Double
Dim m_InitialAngle As Double
Dim m_Payload As Double
Dim m_Fuel As Double


Private Sub UserControl_Resize()
Rocket.Move 0, 0, ScaleWidth, ScaleHeight

End Sub
Public Sub Tick()

If Extender.Container.Ground.Y1 >= Extender.Top + Height Then
    
    ' next alter the vertical position
    Extender.Top = Extender.Top - vv
    ' then the horizontal position (scale by 2 for display purposes)
    Extender.Left = Extender.Left + vh / 4
    ' calculate the acceleration due to the burn ...
    ' ... if there is any fuel left to burn
    ' ... and not forgetting the y co-ordinate direction
    If m > 0 Then
        a = m_BurnRate * m_FuelVelocity / (m + m_Payload)
        ' reduce the rocket's fuel mass by the amount of fuel burnt
        m = m - m_BurnRate
    Else
        a = 0
    End If
    ' alter the vertical speed (allowing for gravity)
    vv = vv + a * Sin(alpha) - g
    ' and the horizontal speed
    vh = vh + a * Cos(alpha)
    ' calculate the new thrust angle
    If vh = 0 Then
        alpha = pi / 2
    Else
        If vv = 0 Then
            alpha = 0
        Else
            alpha = Atn(vv / vh)
        End If
    End If
    
    Rocket.Caption = Format(vv, "##0.0") + Chr(13) _
                   + Format(vh, "##0.0") + Chr(13) _
                   + Format(m, "##0.0") + Chr(13) _
                   + Format(alpha * 57.3, "##0.0")
End If


End Sub

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_FuelVelocity = m_def_FuelVelocity
    m_BurnRate = m_def_BurnRate
    m_InitialAngle = m_def_InitialAngle
    m_Payload = m_def_Payload
    m_Fuel = m_def_Fuel
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    m_FuelVelocity = PropBag.ReadProperty("FuelVelocity", m_def_FuelVelocity)
    m_BurnRate = PropBag.ReadProperty("BurnRate", m_def_BurnRate)
    m_InitialAngle = PropBag.ReadProperty("InitialAngle", m_def_InitialAngle)
    m_Payload = PropBag.ReadProperty("Payload", m_def_Payload)
    m_Fuel = PropBag.ReadProperty("Fuel", m_def_Fuel)
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    Call PropBag.WriteProperty("FuelVelocity", m_FuelVelocity, m_def_FuelVelocity)
    Call PropBag.WriteProperty("BurnRate", m_BurnRate, m_def_BurnRate)
    Call PropBag.WriteProperty("InitialAngle", m_InitialAngle, m_def_InitialAngle)
    Call PropBag.WriteProperty("Payload", m_Payload, m_def_Payload)
    Call PropBag.WriteProperty("Fuel", m_Fuel, m_def_Fuel)
End Sub

Public Function Initialize() As Boolean

' initialise the angle
pi = 4 * Atn(1)
alpha = m_InitialAngle / 180 * pi

vh = 0
vv = 0
m = m_Fuel

Extender.Top = Extender.Container.Ground.Y1 - Height
Extender.Left = Extender.Container.Ground.X1

Rocket.Caption = ""
End Function
Public Property Get FuelVelocity() As Double
    FuelVelocity = m_FuelVelocity
End Property

Public Property Let FuelVelocity(ByVal New_FuelVelocity As Double)
    m_FuelVelocity = New_FuelVelocity
    PropertyChanged "FuelVelocity"
End Property

Public Property Get BurnRate() As Double
    BurnRate = m_BurnRate
End Property

Public Property Let BurnRate(ByVal New_BurnRate As Double)
    m_BurnRate = New_BurnRate
    PropertyChanged "BurnRate"
End Property

Public Property Get InitialAngle() As Double
    InitialAngle = m_InitialAngle
End Property

Public Property Let InitialAngle(ByVal New_InitialAngle As Double)
    m_InitialAngle = New_InitialAngle
    PropertyChanged "InitialAngle"
End Property

Public Property Get Payload() As Double
    Payload = m_Payload
End Property

Public Property Let Payload(ByVal New_Payload As Double)
    m_Payload = New_Payload
    PropertyChanged "Payload"
End Property

Public Property Get Fuel() As Double
    Fuel = m_Fuel
End Property

Public Property Let Fuel(ByVal New_Fuel As Double)
    m_Fuel = New_Fuel
    PropertyChanged "Fuel"
End Property

